import chardet

NULL = 0


def concatenate(source: bytes, dest: bytes, update=False) -> bytes:
    """
    If the files are text, then a newline will be inserted between them,
    otherwise they will be concatenated together
    """

    if update:
        raise NotImplementedError("Can't update in concatenation")

    if source in dest:
        return dest
    else:
        binary = NULL in dest

        dst_encoding = chardet.detect(dest)["encoding"]
        src_encoding = chardet.detect(source)["encoding"]

        if not binary and dst_encoding and src_encoding:
            source_text = source.decode(src_encoding)

            if dest[-1] != b"\n"[0]:
                return dest + b"\n" + source_text.encode(dst_encoding)
            else:
                return dest + source_text.encode(dst_encoding)

    return dest + source



from typing import Callable

from .concatenate import concatenate
from .env import env

Merger = Callable[[bytes, bytes], bytes]


def get(source_type: str, dest_type: str) -> Merger:
    """
    Return the correct file merger
    """

    if source_type == ".env" and dest_type == ".env":
        return env
    else:
        return concatenate



def env(source: bytes, dest: bytes, update=False) -> bytes:
    """
    Merge two env files together
    """
    # this is a bit hairy, need to refactor

    source_lines = source.decode().splitlines()
    dest_lines = dest.decode().splitlines()

    source_vars = dict(
        line.split("=", maxsplit=1) for line in source_lines if "=" in line
    )

    output = []
    used = set()

    for line in dest_lines:
        # If the line is a comment, just add it to the output
        if "=" not in line:
            output.append(line)
            if line in source_lines:
                used.add(line)
            continue

        var = line.split("=")[0]

        # If the variable is in the source file, add it to the output
        if var in source_vars:
            if update:
                output.append(f"{var}={source_vars[var]}")
            else:
                output.append(line)

            del source_vars[var]
            used.add(var)
            continue

        # If the variable is not in the source file, add it to the output
        output.append(line)

    # Add any lines that were not in the destination file
    for line in source_lines:
        if "=" in line:
            var = line.split("=", maxsplit=1)[0]
            if var not in used:
                output.append(line)
        else:
            if line not in used:
                output.append(line)

    return "\n".join(output).encode()



#!/usr/bin/env python3
"""Merges the source file into the destination file.

Supports .env, text and binary files.
"""
import argparse
import sys
from typing import List

from .mergers import get


def parse_args(command_line: List[str]) -> argparse.Namespace:
    """
    Parse the command line arguments, or a list of strings.

    Might call sys.exit() if the arguments are invalid.
    """
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    parser.add_argument("source", help="The source file")
    parser.add_argument("dest", help="The destination file")
    parser.add_argument(
        "--update", help="Update values?", default=False, action="store_true"
    )

    return parser.parse_args(command_line)


def main():
    """
    Main entry point for the merge-files command line tool.
    """
    args = parse_args(sys.argv[1:])

    with open(args.source, "rb") as source_file:
        source = source_file.read()

    with open(args.dest, "rb") as dest_file:
        dest = dest_file.read()

    merger = get(source, dest)
    output_data = merger(source, dest, args.update)

    with open(args.dest, "wb") as dest_file:
        dest_file.write(output_data)


if __name__ == "__main__":
    main()
